from math import pi
from io import BytesIO
import base64
from django.shortcuts import render
from django.utils.safestring import mark_safe
import re

from bokeh.plotting import figure as bokehfig
from bokeh.embed import components
from bokeh.models import Label, Node, MathML


from isotopapp.models import Isotope
# Create your views here.

def home(request):
    isotopes = [i for i in Isotope.objects.all() if (i.gamma != 0 or i.stable)]
    return render(request, 'home.html', {'isotopes': [[f"{i.n_nucleons}{i.symbol}", mark_safe(f"<sup>{i.n_nucleons}</sup>{i.symbol}")] for i in isotopes],})

def extract_isotope_parts(isotope_str):
    """Extracts the number and element from an isotope string (e.g., '23Na')."""
    print(isotope_str)
    match = re.match(r"(\d+)([A-Za-z]+)", isotope_str)
    if not match:
        raise ValueError("Invalid isotope format")
    return int(match.group(1)), match.group(2)

def isotope_info(isotope, field):
    f_larmor = field*isotope.gamma
    if isotope.spin_quantum_number.as_integer_ratio()[1]==1:
        spin = int(isotope.spin_quantum_number)
    else:
        spin = mark_safe(f"<sup>{isotope.spin_quantum_number.as_integer_ratio()[0]}</sup>&frasl;<sub>{isotope.spin_quantum_number.as_integer_ratio()[1]}</sub>")
    return [isotope.n_nucleons,
            mark_safe(f"<sup>{isotope.n_nucleons}</sup>{isotope.symbol}"),
            isotope.name.capitalize(),
            f"{f_larmor:.3f}",
            spin,
            f"{isotope.natural_abundance:.1f}",
            f"{isotope.gamma:.5f}"]

def result(request):
    n1, element1 = extract_isotope_parts(request.GET.get('isotope1'))
    isotope1 = Isotope.objects.filter(symbol=element1, n_nucleons=n1).get()

    freq = float(request.GET.get('freq'))

    field_T = freq / isotope1.gamma

    script = None
    div = None

    print(request.GET)
    if request.GET.get('range_search') == "":
        close_isotopes = [isotope_info(isotope1, field_T)]
        freq_range = float(request.GET.get('freq_range'))
        Isotope.objects.filter()
        # calculate the frequency for all isotopes and compile a list of close by isotopes
        for isotope in Isotope.objects.all():
            if isotope.gamma == 0: continue
            if not isotope.stable: continue
            f_Larmor = field_T*isotope.gamma
            if abs(f_Larmor - freq) <= freq_range:
                close_isotopes.append(isotope_info(isotope, field_T))
        ans = sorted(close_isotopes, key=lambda x: -float(x[3]))

    elif request.GET.get('gradient_search') == "":
        sample_diameter = 5e-3
        gradient = float(request.GET.get('gradient'))
        # create a plot (bokeh)
        plot = bokehfig(outer_width=400, outer_height=400, match_aspect=True)
        plot.ellipse(x=[0], y=[0], width=5, height=5, color="#D5D9FF", alpha=0.8, line_width=1, line_color="black")
        plot.ellipse(x=[0], y= [2.5], width=5, height=5, color="#D5D9FF", alpha=0.4, line_width=1, line_color="black")
        plot.ellipse(x=[0], y=[-2.5], width=5, height=5, color="#D5D9FF", alpha=0.4, line_width=1, line_color="black")


        close_isotopes = []
        for isotope in Isotope.objects.all():
            if isotope.gamma == 0: continue
            if not isotope.stable: continue
            z = (freq/isotope.gamma-field_T)/gradient
            if abs(z) <= sample_diameter:
                i_info = isotope_info(isotope, field_T)
                #i_info[3] = f"{z*1e3:.1f} mm"
                close_isotopes.append(i_info)
                mathml = f"""
                <math>
                <msup>
                {isotope.n_nucleons}
                </msup>
                {isotope.symbol}
                </math>
                """
                plot.rect(x=[0], y=[z*1e3], width=5, height=0.2, color="black", alpha=0.6)
                label = Label(x=2.6, y=z*1e3, text=f"{isotope.n_nucleons}{isotope.symbol}", text_baseline="middle", text_align="left", text_font_size="16pt")
                #label = Label(x=2.6, y=z*1e3, text=MathML(mathml), text_baseline="middle", text_align="left", text_font_size="16pt")
                plot.add_layout(label)

        frame_left = Node(target="frame", symbol="left", offset=5)
        frame_bottom = Node(target="frame", symbol="bottom", offset=-5)
        citation = Label(
            x=frame_left,
            y=frame_bottom,
            anchor="bottom_left",
            #text=MathML(text=f"<math><msup>{isotope1.n_nucleons}</msup>{isotope1.symbol}: {freq:.1f} MHz\ng={gradient:.1f} T/m\n5 mm sample dia.</math>"),
            text=f"{isotope1.n_nucleons}{isotope1.symbol}: {freq:.1f} MHz\ng={gradient:.1f} T/m\n5 mm sample dia.",
            #text=MathML("<math>tewt</math>"),
            padding=10,
            border_radius=5,
            border_line_color="black", background_fill_color="white",
        )
        plot.add_layout(citation)

        # boke plot
        script, div = components(plot)
        ans = sorted(close_isotopes, key=lambda x: float(x[3]))

    elif request.GET.get('transform') == "":
        n2, element2 = extract_isotope_parts(request.GET.get('isotope2'))
        isotope2 = Isotope.objects.filter(symbol=element2, n_nucleons=n2).get()
        #isotope_info(isotope2, field_T)
        ans = [isotope_info(isotope2, field_T)]
    else:
        ans = []
    return render(request, 'result.html', {'ans': ans, 'script': script, 'div': div})
